<?php
/*
class-share-twitter-strategy.php

Description: This class is abstract class of a data crawler
Author: Daisuke Maruyama
Author URI: http://marubon.info/
License: GPL2 or later
License URI: http://www.gnu.org/licenses/gpl-2.0.txt
*/

/*
Copyright (C) 2014 - 2016 Daisuke Maruyama

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

class Share_Twitter_Strategy extends Crawl_Strategy {

	/**
	 * SNS base url
	 */
	const DEF_BASE_URL_JSOON = 'http://jsoon.digitiminimi.com/twitter/count.json';

	const DEF_BASE_URL_OPENSHARECOUNT = 'http://opensharecount.com/count.json';

	const DEF_BASE_URL_TWITCOUNT = 'http://opensharecount.com/count.json';

	/**
	 * Class constarctor
	 * Hook onto all of the actions and filters needed by the plugin.
	 *
	 */
	protected function __construct() {
		Common_Util::log('[' . __METHOD__ . '] (line='. __LINE__ . ')');
		$this->method = 'GET';
	}

	/**
	 * Initialization
	 *
	 * @since 0.9.0
	 */
	public function initialize( $options = array() ) {
		Common_Util::log( '[' . __METHOD__ . '] (line='. __LINE__ . ')' );

		if ( isset( $options['url'] ) ) $this->url = $options['url'];
		if ( isset( $options['method'] ) ) $this->method = $options['method'];
		if ( isset( $options['parameters'] ) ) $this->parameters = $options['parameters'];
		if ( isset( $options['twitter_api'] ) ) $this->twitter_api = $options['twitter_api'];
	}

	/**
	 * Build header
	 *
	 * @since 0.9.0
	 */
	public function build_header() {
		Common_Util::log( '[' . __METHOD__ . '] (line='. __LINE__ . ')' );
		return null;
	}

	/**
	 * Build query url
	 *
	 * @since 0.9.0
	 */
	public function build_query_url() {
		Common_Util::log( '[' . __METHOD__ . '] (line='. __LINE__ . ')' );

		$base_url = self::DEF_BASE_URL_JSOON;

		if ( isset( $this->twitter_api ) && $this->twitter_api ) {
			if ( $this->twitter_api === SNS_Count_Cache::OPT_SHARE_TWITTER_API_JSOON ) {
				$base_url = self::DEF_BASE_URL_JSOON;
			} elseif ( $this->twitter_api ===  SNS_Count_Cache::OPT_SHARE_TWITTER_API_OPENSHARECOUNT ) {
				$base_url = self::DEF_BASE_URL_OPENSHARECOUNT;
			} elseif( $this->twitter_api === SNS_Count_Cache::OPT_SHARE_TWITTER_API_TWITCOUNT ) {
				$base_url = self::DEF_BASE_URL_TWITCOUNT;
			}
		}

		$url = $base_url . '?' . http_build_query( $this->query_parameters , '' , '&' );

		return $url;
	}

	/**
	 * Extract count
	 *
	 * @since 0.9.0
	 */
	public function extract_count( $content ) {
		Common_Util::log( '[' . __METHOD__ . '] (line='. __LINE__ . ')' );

		$count = (int) -1;

		if ( isset( $content['data'] ) && empty( $content['error'] ) ) {
			$json = json_decode( $content['data'], true );

			if ( isset( $json['count'] ) && is_numeric( $json['count'] ) ) {
				$count = (int) $json['count'];
			} else {
				$count = (int) -1;
			}
		} else {
			$count = (int) -1;
		}

		return $count;
	}

	public function set_query_parameter( $key, $value ) {
		if ( isset( $this->twitter_api ) && $this->twitter_api ) {
			if ( $this->twitter_api === SNS_Count_Cache::OPT_SHARE_TWITTER_API_JSOON ) {
				if ( $key === 'url' ) {
					if ( $value ===  home_url( '/', 'http' ) || $value ===  home_url( '/', 'https' ) ) {
						$this->query_parameters[$key] = '"' . $value . '"';
					} else {
						$this->query_parameters[$key] = $value;
					}
				} else {
					$this->query_parameters[$key] = $value;
				}
			} elseif ( $this->twitter_api ===  SNS_Count_Cache::OPT_SHARE_TWITTER_API_OPENSHARECOUNT ) {
				$this->query_parameters[$key] = $value;
			} elseif( $this->twitter_api === SNS_Count_Cache::OPT_SHARE_TWITTER_API_TWITCOUNT ) {
				$this->query_parameters[$key] = $value;
			}
		} else {
			$this->query_parameters[$key] = $value;
		}
	}

	/**
	 * Check if required paramters are included or not.
	 *
	 * @since 0.9.0
	 */
	public function check_configuration() {
		Common_Util::log( '[' . __METHOD__ . '] (line='. __LINE__ . ')' );

		if ( isset( $this->query_parameters['url'] ) && $this->query_parameters['url'] ) {
			return true;
		} else {
			return false;
		}
	}

}
